@@ -1,5 +1,4 @@
-from __future__ import absolute_import
-
+# ruff: noqa: B008
 import collections
 import opcode
 import os
@@ -14,10 +13,9 @@ from .util import CALL_COLORS
 from .util import CODE_COLORS
 from .util import MISSING
 from .util import OTHER_COLORS
-from .util import PY3
-from .util import StringType
 from .util import builtins
 from .util import frame_iterator
+from .util import get_arguments
 from .util import iter_symbols
 from .util import safe_repr
 from colorama import AnsiToWin32
@@ -27,20 +25,24 @@ try:
 except ImportError:
     from thread import get_ident
 
-__all__ = ['Action', 'Debugger', 'Manhole', 'CodePrinter', 'CallPrinter', 'VarsPrinter']
+__all__ = [
+    'Action',
+    'Debugger',
+    'Manhole',
+    'CodePrinter',
+    'CallPrinter',
+    'VarsPrinter',
+]
 
-BUILTIN_REPR_FUNCS = {
-    'repr': repr,
-    'safe_repr': safe_repr
-}
+BUILTIN_REPR_FUNCS = {'repr': repr, 'safe_repr': safe_repr}
 
 
-class Action(object):
+class Action:
     def __call__(self, event):
-        raise NotImplementedError()
+        raise NotImplementedError
 
 
-class LazyImportPdb(object):
+class LazyImportPdb:
     def __repr__(self):
         return "<class 'pdb.Pdb'>"
 
@@ -48,6 +50,7 @@ class LazyImportPdb(object):
 
     def __call__(self, **kwargs):
         from pdb import Pdb
+
         return Pdb(**kwargs)
 
 
@@ -61,11 +64,7 @@ class Debugger(Action):
         self.kwargs = kwargs
 
     def __eq__(self, other):
-        return (
-            type(self) is type(other) and
-            self.klass == other.klass and
-            self.kwargs == other.kwargs
-        )
+        return type(self) is type(other) and self.klass == other.klass and self.kwargs == other.kwargs
 
     def __str__(self):
         return '{0.__class__.__name__}(klass={0.klass}, kwargs={0.kwargs})'.format(self)
@@ -95,6 +94,7 @@ class Manhole(Action):
 
     def __call__(self, event):
         import manhole
+
         inst = manhole.install(strict=False, thread=False, **self.options)
         inst.handle_oneshot()
 
@@ -103,6 +103,7 @@ class ColorStreamAction(Action):
     """
     Baseclass for your custom action. Just implement your own ``__call__``.
     """
+
     _stream_cache = {}
     _stream = None
     _tty = None
@@ -111,15 +112,17 @@ class ColorStreamAction(Action):
     OTHER_COLORS = OTHER_COLORS
     EVENT_COLORS = CODE_COLORS
 
-    def __init__(self,
-                 stream=config.Default('stream', None),
-                 force_colors=config.Default('force_colors', False),
-                 force_pid=config.Default('force_pid', False),
-                 filename_alignment=config.Default('filename_alignment', 40),
-                 thread_alignment=config.Default('thread_alignment', 12),
-                 pid_alignment=config.Default('pid_alignment', 9),
-                 repr_limit=config.Default('repr_limit', 1024),
-                 repr_func=config.Default('repr_func', 'safe_repr')):
+    def __init__(
+        self,
+        stream=config.Default('stream', None),
+        force_colors=config.Default('force_colors', False),
+        force_pid=config.Default('force_pid', False),
+        filename_alignment=config.Default('filename_alignment', 40),
+        thread_alignment=config.Default('thread_alignment', 12),
+        pid_alignment=config.Default('pid_alignment', 9),
+        repr_limit=config.Default('repr_limit', 1024),
+        repr_func=config.Default('repr_func', 'safe_repr'),
+    ):
         self.force_colors = config.resolve(force_colors)
         self.force_pid = config.resolve(force_pid)
         stream = config.resolve(stream)
@@ -147,16 +150,20 @@ class ColorStreamAction(Action):
         )
 
     def __str__(self):
-        return '{0.__class__.__name__}(stream={0.stream}, force_colors={0.force_colors}, ' \
-               'filename_alignment={0.filename_alignment}, thread_alignment={0.thread_alignment}, ' \
-               'pid_alignment={0.pid_alignment} repr_limit={0.repr_limit}, ' \
-               'repr_func={0.repr_func})'.format(self)
+        return (
+            '{0.__class__.__name__}(stream={0.stream}, force_colors={0.force_colors}, '
+            'filename_alignment={0.filename_alignment}, thread_alignment={0.thread_alignment}, '
+            'pid_alignment={0.pid_alignment} repr_limit={0.repr_limit}, '
+            'repr_func={0.repr_func})'.format(self)
+        )
 
     def __repr__(self):
-        return '{0.__class__.__name__}(stream={0.stream!r}, force_colors={0.force_colors!r}, ' \
-               'filename_alignment={0.filename_alignment!r}, thread_alignment={0.thread_alignment!r}, ' \
-               'pid_alignment={0.pid_alignment!r} repr_limit={0.repr_limit!r}, ' \
-               'repr_func={0.repr_func!r})'.format(self)
+        return (
+            '{0.__class__.__name__}(stream={0.stream!r}, force_colors={0.force_colors!r}, '
+            'filename_alignment={0.filename_alignment!r}, thread_alignment={0.thread_alignment!r}, '
+            'pid_alignment={0.pid_alignment!r} repr_limit={0.repr_limit!r}, '
+            'repr_func={0.repr_func!r})'.format(self)
+        )
 
     @property
     def stream(self):
@@ -164,7 +171,7 @@ class ColorStreamAction(Action):
 
     @stream.setter
     def stream(self, value):
-        if isinstance(value, StringType):
+        if isinstance(value, str):
             if value in self._stream_cache:
                 value = self._stream_cache[value]
             else:
@@ -193,7 +200,7 @@ class ColorStreamAction(Action):
         elif value in BUILTIN_REPR_FUNCS:
             self._repr_func = BUILTIN_REPR_FUNCS[value]
         else:
-            raise TypeError('Expected a callable or either "repr" or "safe_repr" strings, not {!r}.'.format(value))
+            raise TypeError(f'Expected a callable or either "repr" or "safe_repr" strings, not {value!r}.')
 
     def try_str(self, obj):
         """
@@ -248,8 +255,9 @@ class ColorStreamAction(Action):
         elif source:
             return source
         else:
-            return '{SOURCE-FAILURE}??? NO SOURCE: {SOURCE-DETAIL}Source code string for module {!r} is empty.'.format(
-                event.module, **self.other_colors)
+            return '{SOURCE-FAILURE}??? NO SOURCE: {SOURCE-DETAIL}Source code string for {!r} is empty.'.format(
+                event.filename, **self.other_colors
+            )
 
     def filename_prefix(self, event=None):
         """
@@ -258,16 +266,14 @@ class ColorStreamAction(Action):
         Returns: string
         """
         if event:
-            if event.builtin:
-                filename = '<{}>'.format(event.module)
-                lineno = '      '
-            else:
-                filename = event.filename or '<???>'
-                lineno = '{COLON}:{LINENO}{:<5}'.format(event.lineno, **self.other_colors)
+            filename = event.filename
+            if filename == '?':
+                filename = '{SOURCE-FAILURE}?'.format(**self.other_colors)
+            lineno = '{COLON}:{LINENO}{:<5}'.format(event.lineno, **self.other_colors)
+
             if len(filename) > self.filename_alignment:
-                filename = '[...]{}'.format(filename[5 - self.filename_alignment:])
-            return '{:>{}}{} '.format(
-                filename, self.filename_alignment, lineno, **self.other_colors)
+                filename = f'[...]{filename[5 - self.filename_alignment :]}'
+            return '{:>{}}{} '.format(filename, self.filename_alignment, lineno, **self.other_colors)
         else:
             return '{:>{}}       '.format('', self.filename_alignment)
 
@@ -277,7 +283,7 @@ class ColorStreamAction(Action):
         """
         pid = getpid()
         if self.force_pid or self.seen_pid != pid:
-            pid = '[{}]'.format(pid)
+            pid = f'[{pid}]'
             pid_align = self.pid_alignment
         else:
             pid = pid_align = ''
@@ -334,10 +340,7 @@ class ColorStreamAction(Action):
 
         Returns: string
         """
-        self.stream.write(format_str.format(
-            *args,
-            **dict(self.other_colors, **kwargs)
-        ))
+        self.stream.write(format_str.format(*args, **dict(self.other_colors, **kwargs)))
 
     def cleanup(self):
         pass
@@ -425,6 +428,7 @@ class CallPrinter(CodePrinter):
 
     .. versionadded:: 1.2.0
     """
+
     EVENT_COLORS = CALL_COLORS
 
     locals = defaultdict(list)
@@ -434,7 +438,7 @@ class CallPrinter(CodePrinter):
         cls.locals = defaultdict(list)
 
     def __init__(self, *args, **kwargs):
-        super(CallPrinter, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def __call__(self, event):
         """
@@ -450,65 +454,97 @@ class CallPrinter(CodePrinter):
         thread_prefix = self.thread_prefix(event)
         filename_prefix = self.filename_prefix(event)
 
-        if event.builtin and event.kind == 'call':
-            stack.append(ident)
-            self.output(
-                '{}{}{}{KIND}{:9} {}{COLOR}=>{NORMAL} {}: {}{RESET}\n',
-                pid_prefix,
-                thread_prefix,
-                filename_prefix,
-                event.kind,
-                '   ' * (len(stack) - 1),
-                event.function,
-                self.try_source(event).strip(),
-                COLOR=self.event_colors.get(event.kind),
-            )
-        elif event.kind == 'call':
-            code = event.code
-            stack.append(ident)
-            self.output(
-                '{}{}{}{KIND}{:9} {}{COLOR}=>{NORMAL} {}({}{COLOR}{NORMAL}){RESET}\n',
-                pid_prefix,
-                thread_prefix,
-                filename_prefix,
-                event.kind,
-                '   ' * (len(stack) - 1),
-                event.function,
-                ', '.join('{VARS}{VARS-NAME}{0}{VARS}={RESET}{1}'.format(
-                    var,
-                    self.try_str(event.locals.get(var, MISSING)) if event.detached else self.try_repr(event.locals.get(var, MISSING)),
-                    **self.other_colors
-                ) for var in code.co_varnames[:code.co_argcount]),
-                COLOR=self.event_colors.get(event.kind),
-            )
+        if event.kind == 'call':
+            if event.builtin:
+                self.output(
+                    '{}{}{}{KIND}{:9} {}{COLOR} >{BUILTIN} {}.{}: {}{RESET}\n',
+                    pid_prefix,
+                    thread_prefix,
+                    filename_prefix,
+                    event.kind,
+                    '   ' * (len(stack) - 1),
+                    event.module,
+                    event.function,
+                    self.try_source(event).strip(),
+                    COLOR=self.event_colors.get(event.kind),
+                )
+            else:
+                code = event.code
+                stack.append(ident)
+                self.output(
+                    '{}{}{}{KIND}{:9} {}{COLOR}=>{NORMAL} {}({}{COLOR}{NORMAL}){RESET}\n',
+                    pid_prefix,
+                    thread_prefix,
+                    filename_prefix,
+                    event.kind,
+                    '   ' * (len(stack) - 1),
+                    event.function,
+                    ', '.join(
+                        '{VARS}{0}{VARS-NAME}{1}{VARS}={RESET}{2}'.format(
+                            prefix,
+                            var_display,
+                            self.try_str(event.locals.get(var_lookup, MISSING))
+                            if event.detached
+                            else self.try_repr(event.locals.get(var_lookup, MISSING)),
+                            **self.other_colors,
+                        )
+                        for prefix, var_lookup, var_display in get_arguments(code)
+                    ),
+                    COLOR=self.event_colors.get(event.kind),
+                )
         elif event.kind == 'exception':
-            self.output(
-                '{}{}{}{KIND}{:9} {}{COLOR} !{NORMAL} {}: {RESET}{}\n',
-                pid_prefix,
-                thread_prefix,
-                filename_prefix,
-                event.kind,
-                '   ' * (len(stack) - 1),
-                event.function,
-                self.try_str(event.arg) if event.detached else self.try_repr(event.arg),
-                COLOR=self.event_colors.get(event.kind),
-            )
-
+            if event.builtin:
+                self.output(
+                    '{}{}{}{KIND}{:9} {}{COLOR} ! {BUILTIN}{}.{}{RESET}\n',
+                    pid_prefix,
+                    thread_prefix,
+                    filename_prefix,
+                    event.kind,
+                    '   ' * (len(stack) - 1),
+                    event.module,
+                    event.function,
+                    COLOR=self.event_colors.get(event.kind),
+                )
+            else:
+                self.output(
+                    '{}{}{}{KIND}{:9} {}{COLOR} !{NORMAL} {}: {RESET}{}\n',
+                    pid_prefix,
+                    thread_prefix,
+                    filename_prefix,
+                    event.kind,
+                    '   ' * (len(stack) - 1),
+                    event.function,
+                    self.try_str(event.arg) if event.detached else self.try_repr(event.arg),
+                    COLOR=self.event_colors.get(event.kind),
+                )
         elif event.kind == 'return':
-            self.output(
-                '{}{}{}{KIND}{:9} {}{COLOR}<={NORMAL} {}{}{RESET}{}\n',
-                pid_prefix,
-                thread_prefix,
-                filename_prefix,
-                event.kind,
-                '   ' * (len(stack) - 1),
-                event.function,
-                '' if event.builtin else ': ',
-                '' if event.builtin else (self.try_str(event.arg) if event.detached else self.try_repr(event.arg)),
-                COLOR=self.event_colors.get(event.kind),
-            )
-            if stack and stack[-1] == ident:
-                stack.pop()
+            if event.builtin:
+                self.output(
+                    '{}{}{}{KIND}{:9} {}{COLOR} <{BUILTIN} {}.{}{RESET}\n',
+                    pid_prefix,
+                    thread_prefix,
+                    filename_prefix,
+                    event.kind,
+                    '   ' * (len(stack) - 1),
+                    event.module,
+                    event.function,
+                    COLOR=self.event_colors.get(event.kind),
+                )
+            else:
+                self.output(
+                    '{}{}{}{KIND}{:9} {}{COLOR}<={NORMAL} {}{}{RESET}{}\n',
+                    pid_prefix,
+                    thread_prefix,
+                    filename_prefix,
+                    event.kind,
+                    '   ' * (len(stack) - 1),
+                    event.function,
+                    '' if event.builtin else ': ',
+                    '' if event.builtin else (self.try_str(event.arg) if event.detached else self.try_repr(event.arg)),
+                    COLOR=self.event_colors.get(event.kind),
+                )
+                if stack and stack[-1] == ident:
+                    stack.pop()
         else:
             self.output(
                 '{}{}{}{KIND}{:9} {RESET}{}{}{RESET}\n',
@@ -539,11 +575,8 @@ class VarsPrinter(ColorStreamAction):
     def __init__(self, *names, **options):
         if not names:
             raise TypeError('VarsPrinter requires at least one variable name/expression.')
-        self.names = {
-            name: set(iter_symbols(name))
-            for name in names
-        }
-        super(VarsPrinter, self).__init__(**options)
+        self.names = {name: set(iter_symbols(name)) for name in names}
+        super().__init__(**options)
 
     def __call__(self, event):
         """
@@ -613,7 +646,7 @@ class VarsSnooper(ColorStreamAction):
     """
 
     def __init__(self, **options):
-        super(VarsSnooper, self).__init__(**options)
+        super().__init__(**options)
         self.stored_reprs = defaultdict(dict)
 
     def __call__(self, event):
@@ -627,10 +660,7 @@ class VarsSnooper(ColorStreamAction):
         filename_prefix = self.filename_prefix(event)
         empty_filename_prefix = self.filename_prefix()
 
-        current_reprs = {
-            name: self.try_str(value) if event.detached else self.try_repr(value)
-            for name, value in event.locals.items()
-        }
+        current_reprs = {name: self.try_str(value) if event.detached else self.try_repr(value) for name, value in event.locals.items()}
         scope_key = event.code or event.function
         scope = self.stored_reprs[scope_key]
         for name, current_repr in sorted(current_reprs.items()):
@@ -684,12 +714,11 @@ class VarsSnooper(ColorStreamAction):
                         current_repr,
                     )
         if event.kind == 'return':
-            del self.stored_reprs[scope_key]
+            if scope_key in self.stored_reprs:
+                del self.stored_reprs[scope_key]
 
 
 RETURN_VALUE = opcode.opmap['RETURN_VALUE']
-if not PY3:
-    RETURN_VALUE = chr(RETURN_VALUE)
 
 
 class ErrorSnooper(CodePrinter):
@@ -733,7 +762,7 @@ class ErrorSnooper(CodePrinter):
         self.max_depth = kwargs.pop('max_depth', 0)
         self.origin = None
         self.events = None
-        super(ErrorSnooper, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def __call__(self, event):
         if self.max_backlog:
@@ -760,7 +789,10 @@ class ErrorSnooper(CodePrinter):
                 if event.depth == self.origin.depth - 1:  # stop if the same function returned (depth is -1)
                     if event.instruction == RETURN_VALUE:
                         self.dump_events()
-                        self.output('{BRIGHT}{fore(BLACK)}{} function exit{RESET}\n', '-' * 46)
+                        self.output(
+                            '{BRIGHT}{fore(BLACK)}{} function exit{RESET}\n',
+                            '-' * 46,
+                        )
                     self.origin = None
                     self.events = None
             elif event.depth > self.origin.depth + self.max_depth:  # too many details
@@ -774,10 +806,14 @@ class ErrorSnooper(CodePrinter):
                 self.events.append(detached_event)
 
     def dump_events(self):
-        self.output('{BRIGHT}{fore(BLUE)}{} tracing {fore(YELLOW)}{}{fore(BLUE)} on {fore(RED)}{}{RESET}\n',
-                    '>' * 46, self.origin.function, self.origin.arg)
+        self.output(
+            '{BRIGHT}{fore(BLUE)}{} tracing {fore(YELLOW)}{}{fore(BLUE)} on {fore(RED)}{}{RESET}\n',
+            '>' * 46,
+            self.origin.function,
+            self.origin.arg,
+        )
         for event in self.events:
-            super(ErrorSnooper, self).__call__(event)
+            super().__call__(event)
         self.origin = None
         self.events = None
 
@@ -800,7 +836,7 @@ class StackPrinter(ColorStreamAction):
     def __init__(self, depth=15, limit=2, **options):
         self.limit = limit
         self.depth = depth
-        super(StackPrinter, self).__init__(**options)
+        super().__init__(**options)
 
     def __call__(self, event):
         """
@@ -812,19 +848,19 @@ class StackPrinter(ColorStreamAction):
         sep = os.path.sep
 
         if event.frame and event.frame.f_back:
-            template = '{}{}{}{CONT}:{BRIGHT}{fore(BLUE)}%s {KIND}<={RESET} %s' % (
+            template = '{{}}{{}}{{}}{{CONT}}:{{BRIGHT}}{{fore(BLUE)}}{} {{KIND}}<={{RESET}} {}'.format(
                 event.function,
                 ' {KIND}<={RESET} '.join(
-                    '%s{CONT}:{RESET}%s{CONT}:{BRIGHT}{fore(BLUE)}%s' % (
-                        '/'.join(frame.f_code.co_filename.split(sep)[-self.limit:]),
+                    '{}{{CONT}}:{{RESET}}{}{{CONT}}:{{BRIGHT}}{{fore(BLUE)}}{}'.format(
+                        '/'.join(frame.f_code.co_filename.split(sep)[-self.limit :]),
                         frame.f_lineno,
-                        frame.f_code.co_name
+                        frame.f_code.co_name,
                     )
                     for frame in islice(frame_iterator(event.frame.f_back), self.depth)
-                )
+                ),
             )
         else:
-            template = '{}{}{}{CONT}:{BRIGHT}{fore(BLUE)}%s {KIND}<= {BRIGHT}{fore(YELLOW)}no frames available {NORMAL}(detached=%s)' % (
+            template = '{{}}{{}}{{}}{{CONT}}:{{BRIGHT}}{{fore(BLUE)}}{} {{KIND}}<= {{BRIGHT}}{{fore(YELLOW)}}no frames available {{NORMAL}}(detached={})'.format(
                 event.function,
                 event.detached,
             )
